/*
 * Created on 25.10.2005
 *
 */
package org.eclipse.contribution.junit;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.DocumentEvent;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IDocumentListener;

/**
 * @author Andreas
 *
 */
public class JUnitDocumentListener implements IDocumentListener{

	private IMarker[] testmarkers;
	private IDocument doc;
	private int currentlinenumbers = -2;
	private IFile file;
	
	public JUnitDocumentListener(IDocument doc, IMarker[] testmarkers, IMarker[] testedmarkers, IFile file){
		this.doc = doc;
		this.testmarkers = testmarkers;
		if (currentlinenumbers==-2)
			currentlinenumbers = doc.getNumberOfLines();
		this.file = file;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.text.IDocumentListener#documentAboutToBeChanged(org.eclipse.jface.text.DocumentEvent)
	 */
	public void documentAboutToBeChanged(DocumentEvent event) {		
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.text.IDocumentListener#documentChanged(org.eclipse.jface.text.DocumentEvent)
	 */
	public void documentChanged(DocumentEvent event) {
		int offset;
		int charstart, charend;
		IMarker marker;
		
		if (doc == null)
			return;
		
		offset = event.getOffset();
				
		//dirtyMarker will be set. The markerCreator can verify if the method was changed!
		try {
			marker = file.createMarker(JUnitPlugin.DIRTY_MARKER);
			marker.setAttribute("org.eclipse.contribution.junit.offset", new Integer(offset));
			setTestMarkers(file.findMarkers(JUnitPlugin.TEST_MARKER, false, IResource.DEPTH_ZERO));setTestMarkers(file.findMarkers(JUnitPlugin.TEST_MARKER, false, IResource.DEPTH_ZERO));
			setTestedMarkers(file.findMarkers(JUnitPlugin.TESTED_MARKER, false, IResource.DEPTH_ZERO));
		} catch (CoreException e1) {
			e1.printStackTrace();
		}
			
		//deletes all testmarkers in the changed method
		for (int i = 0; i<testmarkers.length; i++){
			try {
				charstart = testmarkers[i].getAttribute("org.eclipse.contribution.junit.charstart", -1);
				charend = testmarkers[i].getAttribute("org.eclipse.contribution.junit.charend", -1);
				if ((offset >= charstart &&
						offset <= charend) || charstart == -1){//-1 indicates constructor!
					testmarkers[i].delete();
				}
			} catch (CoreException e) {
				e.printStackTrace();
			}
		}
		//updates all linenumbers below the actual changed linenumber
		if (currentlinenumbers > doc.getNumberOfLines()){
			try {
				//line was deleted
				createLineChangeMarker(doc.getLineOfOffset(event.getOffset()), -1);
			} catch (BadLocationException e) {
				e.printStackTrace();
			}
			currentlinenumbers = doc.getNumberOfLines();
		}else if (currentlinenumbers < doc.getNumberOfLines()){
			try {
				//line was added
				createLineChangeMarker(doc.getLineOfOffset(event.getOffset()), 1);
			} catch (BadLocationException e) {
				e.printStackTrace();
			}
			currentlinenumbers = doc.getNumberOfLines();
		}
	}

	/**
	 * @param markers
	 */
	private void setTestedMarkers(IMarker[] testedmarkers) {
	}

	/**
	 * @param linenumber
	 * @param offset
	 */
	private synchronized void createLineChangeMarker(int linenumber, int offset) {
		IMarker linechangemarker;
		
		try {
			linechangemarker = file.createMarker(JUnitPlugin.LINECHANGE_MARKER);
			linechangemarker.setAttribute("org.eclipse.contribution.junit.offset", new Integer(offset));
			linechangemarker.setAttribute("org.eclipse.contribution.junit.changedlinenumber", new Integer(linenumber));
		} catch (CoreException e) {
			e.printStackTrace();
		}
	}

	/**
	 * @param testmarkers
	 */
	public void setTestMarkers(IMarker[] testmarkers) {
		this.testmarkers = testmarkers;
	}

	/**
	 * @param doc
	 */
	public void setDocument(IDocument doc) {
		this.doc = doc;
		if (currentlinenumbers==-2)
			currentlinenumbers = doc.getNumberOfLines();
	}

}
