/*
 * Created on 17.03.2005
 *
 */
package org.eclipse.contribution.junit;

import java.io.IOException;
import java.io.PrintWriter;
import java.net.Socket;
import java.net.UnknownHostException;
import junit.framework.AssertionFailedError;
import junit.framework.Test;
import junit.framework.TestListener;
import junit.framework.TestResult;
import junit.framework.TestSuite;

/**
 * @author Andreas
 *
 */
public class SocketTestRunner implements TestListener{
	private int port;
	private Socket socket;
	private PrintWriter writer;

	public static void main(String[] args){
		new SocketTestRunner().runTests(args);
	}
	
	/**
	 * @param args
	 */
	private void runTests(String[] args) {
		port = Integer.parseInt(args[0]);
		openClientSocket();
		try{
			TestSuite suite = new TestSuite();
			for (int i = 1; i < args.length; i++)
				suite.addTestSuite(Class.forName(args[i]));
			writer.println("starting tests "+suite.countTestCases());
			TestResult result = new TestResult();
			result.addListener(this);
			suite.run(result);
			
			writer.println("ending tests");
		} catch (ClassNotFoundException e){
			System.out.println("Not a class: "+args[1]);
		} finally {
			closeClientSocket();
		}
	}

	/**
	 * 
	 */
	private void closeClientSocket() {
		writer.close();
		try{
			socket.close();
		} catch (IOException e){
			e.printStackTrace();
		} catch (Exception e){
			e.printStackTrace();
		}
	}

	/**
	 * 
	 */
	private void openClientSocket() {
		for (int i = 0; i < 10; i++){
			try{
				socket = new Socket("localhost", port);
				writer = new PrintWriter(socket.getOutputStream(), true);
				return;
			} catch (UnknownHostException e){
				e.printStackTrace();
			} catch (IOException e){
				e.printStackTrace();
			}
			try {
				Thread.sleep(500);
			} catch (InterruptedException e1){
				e1.printStackTrace();
			}
		}
	}
	

	/* (non-Javadoc)
	 * @see junit.framework.TestListener#addError(junit.framework.Test, java.lang.Throwable)
	 */
	public void addError(Test test, Throwable t) {
		writer.println("failing test " + test);
		t.printStackTrace(writer);
		writer.println("END TRACE");
	}

	/* (non-Javadoc)
	 * @see junit.framework.TestListener#addFailure(junit.framework.Test, junit.framework.AssertionFailedError)
	 */
	public void addFailure(Test test, AssertionFailedError t) {
		addError(test, t);
	}

	/* (non-Javadoc)
	 * @see junit.framework.TestListener#endTest(junit.framework.Test)
	 */
	public void endTest(Test test) {
		writer.println("ending test "+test);
	}

	/* (non-Javadoc)
	 * @see junit.framework.TestListener#startTest(junit.framework.Test)
	 */
	public void startTest(Test test) {
		writer.println("starting test "+test);
	}

}
